<?php 

class blog {
	private $entries;

	public function __construct($useIndex = true, $showNotPublished = false){
		if (is_numeric($useIndex) && is_numeric($showNotPublished)) {
			$start = $useIndex;
			$end = $showNotPublished;
			
			$this->entries = $this->listRange($start, $end);
		} else if (is_numeric($useIndex)) {
			$id = $useIndex;
			
			$this->entries = $this->listOne($id);
		}else {
			$this->entries = $this->listAll($useIndex, $showNotPublished);
		}
	}


/******************************************************************************/
/*																																						*/
/*																	ENTRIES																		*/
/*																																						*/
/******************************************************************************/

	public function getEntries(){
		return $this->entries;
	}
	
	public function hasNextEntries($page) {
		return $page > 1;
	}
	
	public function hasPreviousEntries($lastEntry) {
		$previousEntry = $this->listRange($lastEntry, $lastEntry+1);
		return count($previousEntry) == 1;
	}

	public function hasEntry($id) {
		foreach($this->entries as $entry){
			if($entry->getId() == $id) return true;
		}
		return false;
	}
	public function getEntry($id){
		foreach ($this->entries as $entry) {
			if ($entry->getId() == $id) {
				return $entry;
			}
		}
		return null;
	}
	public function addEntry($obj){
		global $coreConf;
		if($id = $this->saveEntry($obj)){
			$obj->setId($id);
			$this->entries[] = $obj;
		}
		include(BLOG_PLUGINPATH.'atom.php');
	}
	public function delEntry($id){
		foreach ($this->entries as $k=>$obj) {
			if ($obj->getId() == $id && $this->_delEntry($id)) {
				blog::updateIndex();
				unset($this->entries[$k]);
			}
		}
	}


/******************************************************************************/
/*																																						*/
/*																	COMMENTS																	*/
/*																																						*/
/******************************************************************************/
	
	public function hasComment($entry, $id) {
		$this->populateComments($entry);
		$comments = $this->getEntry($entry)->getComments();
		foreach ($comments as $comment) {
			if ($comment->getId() == $id) {
				return true;
			}
		}
		return false;
	}
	public function getComment($entry, $id) {
		$this->populateComments($entry, true, true);
		$comments = $this->getEntry($entry)->getComments();
		foreach ($comments as $comment) {
			if ($comment->getId() == $id) {
				return $comment;
			}
		}
		return null;
	}
	public function getNbComments($entry) {
		$index = json_decode(file_get_contents(BLOG_DATAPATH.$entry.'/index.json'), true);
		return count($index['published']);
	}
	public function addComment($entry, $obj) {
		if($id = $this->saveComment($entry, $obj)){
			$obj->setId($id);
			$blogEntry = $this->getEntry($entry);
			$blogEntry->setNbComments($this->getNbComments($entry));
			$this->saveEntry($blogEntry);
			$this->populateComments($entry);
		}
	}
	public function delComment($entry, $id) {
		$this->populateComments($entry);
		$blogEntry = $this->getEntry($entry);
		$comments = $blogEntry->getComments();
		foreach ($comments as $comment) {
			if ($comment->getId() == $id && $this->_delComment($entry, $id)) {
				blog::updateIndex($entry);
				$blogEntry->setNbComments($this->getNbComments($entry));
				$this->saveEntry($blogEntry);
				$this->populateComments($entry);
				return;
			}
		}
	}
	
	public function populateComments($id, $useIndex = true, $showNotPublished = false) {
		$entry = $this->getEntry($id);
		$entry->setComments($this->listComments($id, $useIndex, $showNotPublished));
	}
	
	public function clearComments($id) {
		$entry = $this->getEntry($id);
		$entry->setComments(array());
	}


/******************************************************************************/
/*																																						*/
/*																	CATEGORY																	*/
/*																																						*/
/******************************************************************************/
	
	public static function listCategories() {
		$data = array();
		
		$index = json_decode(file_get_contents(BLOG_DATAPATH.'categories.json'), true);
		
		foreach ($index as $category) {
			$data[] = new category($category);
		}
		
		return $data;
	}
	
	public static function addCategory($category) {
		$categories = blog::listCategories();
		
		$id = 0;
		foreach($categories as $cat) {
			if ($cat->getId() >= $id) {
				$id = $cat->getId()+1;
			};
		}
		
		$category->setId($id);
		$categories[] = $category;
		
		blog::updateCategoryIndex($id);
		blog::saveCategories($categories);
	}
	
	public static function getCategory($id) {
		if ($id == -1 || !is_file(BLOG_DATAPATH.'categories/'.$id.'.json')) {
			return "Aucune catégorie";
		} else {
			$rawdata = json_decode(file_get_contents(BLOG_DATAPATH.'categories.json'), true);
			
			foreach ($rawdata as $category) {
				if ($category['id'] == $id) {
					return new category($category);
				}
			}
			return "Aucune catégorie";
		}
	}
	
	public static function deleteCategory($id) {
		$categories = blog::listCategories();
		
		foreach ($categories as $k=>$category) {
			if ($category->getId() == $id) {
				unset($categories[$k]);
				if (is_file(BLOG_DATAPATH.'categories/'.$id.'.json')) {
					unlink(BLOG_DATAPATH.'categories/'.$id.'.json');
				}
			}
		}
		
		blog::saveCategories(array_values($categories));
	}
	
	public static function saveCategories($a) {
		$data = array();
		
		foreach ($a as $cat) {
			$data[] = $cat->toArray();
		}
		
		$data = utilSort2DimArray($data, 'label', 'asc');
		
		file_put_contents(BLOG_DATAPATH.'categories.json', json_encode($data), 0666);
	}
	
	public static function updateCategoryIndex($id) {
		$blog = new blog(true, true);
		$entries = $blog->getEntries();
		$index = array(
			'published' => array(),
			'all' => array()
		);
		
		foreach ($entries as $entry) {
			$entry = $entry->toArray();
			if ($entry['category'] == $id) {
				$index['all'][] = $entry['id'];
				if ($entry['published']) {
					$index['published'][] = $entry['id'];
				}
			}
		}
		
		$categories = blog::listCategories();
		
		foreach ($categories as $category) {
			if ($category->getId() == $id) {
				$category->setNb(count($index['all']));
			}
		}
		
		blog::saveCategories($categories);
		
		if (@file_put_contents(BLOG_DATAPATH.'categories/'.$id.'.json', json_encode($index), 0666)) {
			return true;
		}
		return false;
	}


/******************************************************************************/
/*																																						*/
/*																		INDEX																		*/
/*																																						*/
/******************************************************************************/
	
	public static function updateIndex($id=false) {
		if (!is_numeric($id)) {
			$blog = new blog(false, true);
			$entries = $blog->getEntries();
			$index = array(
				'published' => array(),
				'all' => array()
			);
			
			$data = array();
			
			foreach ($entries as $entry) {
				$data[] = $entry->toArray();
			}
			
			$entries = utilSort2DimArray($data, 'date', 'desc');
		
			foreach ($entries as $entry) {
				$index['all'][] = $entry['id'];
				if ($entry['published']) {
					$index['published'][] = $entry['id'];
				}
			}
		
			if (@file_put_contents(BLOG_DATAPATH.'index.json', json_encode($index), 0666)) {
				return true;
			}
			return false;
		} else {
			$blog = new blog($id);
			$blog->populateComments($id,false,true);
			$entry = $blog->getEntry($id);
			$comments = $entry->getComments();
			$index = array(
				'published' => array(),
				'all' => array()
			);
			
			$data = array();
			
			foreach ($comments as $comment) {
				$data[] = $comment->toArray();
			}
			
			$comments = utilSort2DimArray($data, 'date', 'desc');
			
			foreach ($comments as $comment) {
				$index['all'][] = $comment['id'];
				if ($comment['published']) {
					$index['published'][] = $comment['id'];
				}
			}
			
			if (@file_put_contents(BLOG_DATAPATH.$id.'/index.json', json_encode($index), 0666)) {
				return true;
			}
			return false;
		}
	}
	
	private function listOne($id) {
		$data = array();
		
		if (is_file(BLOG_DATAPATH.$id.'.txt')) {
			$data[] = new blogEntry(json_decode(file_get_contents(BLOG_DATAPATH.$id.'.txt'), true));
		}
		
		return $data;
	}
	
	private function listRange($start, $end) {
		$data = array();
		
		$index = json_decode(file_get_contents(BLOG_DATAPATH.'index.json'), true);
		$index = $index['published'];
		
		for ($i = $start; $i < min($end,count($index)); $i += 1) {
			if (is_file(BLOG_DATAPATH.$index[$i].'.txt')) {
				$data[] = new blogEntry(json_decode(file_get_contents(BLOG_DATAPATH.$index[$i].'.txt'), true));
			}
		}
		
		return $data;
	}

	private function listAll($useIndex, $showNotPublished) {
		$data = array();
		
		if ($useIndex) {
			$index = json_decode(file_get_contents(BLOG_DATAPATH.'index.json'), true);
			
			if ($showNotPublished) {
				foreach ($index['all'] as $entry) {
						$data[] = new blogEntry(json_decode(file_get_contents(BLOG_DATAPATH.$entry.'.txt'), true));
				}
			} else {
				foreach ($index['published'] as $entry) {
						$data[] = new blogEntry(json_decode(file_get_contents(BLOG_DATAPATH.$entry.'.txt'), true));
				}
			}
		} else {
			$dataNotSorted = array();
			$avoid = array('config.txt', 'index.json', 'categories.json');
			$ls = utilScanDir(BLOG_DATAPATH, $avoid);
			foreach ($ls['file'] as $file) {
				$dataNotSorted[] = json_decode(file_get_contents(BLOG_DATAPATH.$file), true);
			}
			$dataSorted = utilSort2DimArray($dataNotSorted, 'id', 'desc');
			foreach ($dataSorted as $val) {
				$data[] = new blogEntry($val);
			}
		}
		return $data;
	}
	
	private function listComments($id, $useIndex = true, $showNotPublished = false) {
		$comments = array();

		if (!is_dir(BLOG_DATAPATH.$id)) {
			return $comments;
		}
		
		if ($useIndex) {
			if (is_file(BLOG_DATAPATH.$id.'/index.json')) {
				$index = json_decode(file_get_contents(BLOG_DATAPATH.$id.'/index.json'), true);
			} else {
				$index = array('all'=>array(),'published'=>array());
			}
			
			if ($showNotPublished) {
				foreach ($index['all'] as $comment) {
						$comments[] = new commentEntry(json_decode(file_get_contents(BLOG_DATAPATH.$id.'/'.$comment.'.txt'), true));
				}
			} else {
				foreach ($index['published'] as $comment) {
						$comments[] = new commentEntry(json_decode(file_get_contents(BLOG_DATAPATH.$id.'/'.$comment.'.txt'), true));
				}
			}
		} else {
			$dataNotSorted = array();
			$avoid = array('index.json');
			$ls = utilScanDir(BLOG_DATAPATH.$id, $avoid);
			foreach ($ls['file'] as $file) {
				$dataNotSorted[] = json_decode(file_get_contents(BLOG_DATAPATH.$id.'/'.$file), true);
			}
		
			$dataSorted = utilSort2DimArray($dataNotSorted, 'id', 'asc');
			foreach ($dataSorted as $val) {
				$comments[] = new commentEntry($val);
			}
		}
		
		return $comments;
	}
	
	private function saveEntry($obj) {
		$id = intval($obj->getId());
		if ($id < 1) {
			$obj->setId($this->makeId());
			$id = intval($obj->getId());
		} else {
			$this->clearComments($id);
		}
		$data = $obj->toArray();
		if (@file_put_contents(BLOG_DATAPATH.$id.'.txt', json_encode($data), 0666)) {
			if (is_dir(BLOG_DATAPATH.$id) || @mkdir(BLOG_DATAPATH.$id)) {
				blog::updateIndex();
				return $id;
			}
		}
		return false;
	}
	
	private function makeId() {
		$ids = array();
		foreach($this->entries as $obj) {
			$ids[] = $obj->getId();
		}
		return @max($ids)+1;
	}
	
	private function _delEntry($id) {
		$ls = utilScanDir(BLOG_DATAPATH.$id);
		foreach ($ls['file'] as $file) {
			if (!@unlink(BLOG_DATAPATH.$id.'/'.$file)) {
				return false;
			}
		}
		
		if (!@rmdir(BLOG_DATAPATH.$id)) {
			return false;
		}
		
		if (!@unlink(BLOG_DATAPATH.$id.'.txt')) {
			return false;
		}
		
		return true;
	}
	
	private function saveComment($entry, $obj) {
		$id = intval($obj->getId());
		if ($id < 1) {
			$obj->setId($this->makeIdComment($entry));
			$id = intval($obj->getId());
		}
		$data = $obj->toArray();
		if (@file_put_contents(BLOG_DATAPATH.$entry.'/'.$id.'.txt', json_encode($data), 0666)) {
			blog::updateIndex($entry);
			return $id;
		}
		return false;
	}
	
	private function makeIdComment($entry) {
		$ids = array();
		$this->populateComments($entry,true,true);
		$comments = $this->getEntry($entry)->getComments();
		foreach ($comments as $comment) {
			$ids[] = $comment->getId();
		}
		return @max($ids)+1;
	}
	
	private function _delComment($entry, $id) {
		if (@unlink(BLOG_DATAPATH.$entry.'/'.$id.'.txt')) {
			return true;
		}
		return false;
	}
}

?>
